{$include lem_directives.inc}
unit AppController;

interface

uses
  Classes, SysUtils, UMisc, //typinfo,
  Forms,
  LemTypes, LemRendering, LemLevel, LemDosStyle, LemDosGraphicSet,
  FBaseDosForm,
  GameControl;

type
  {-------------------------------------------------------------------------------
    The main application screen logic is handled by this class.
    it's a kind of simple statemachine, which shows the appropriate screens.
    These screens must change the GameParams.NextScreen property, when closing.
  -------------------------------------------------------------------------------}
  TAppController = class(TComponent)
  private
    //fCurrentScreen: TBaseDosForm;
    fGameParams: TDosGameParams; // instance
  protected
  public
    constructor Create(aOwner: TComponent); override;
    destructor Destroy; override;

    procedure ShowMenuScreen;
    procedure ShowPreviewScreen;
    procedure ShowPlayScreen;
    procedure ShowPostviewScreen;
    procedure ShowLevelCodeScreen;
    procedure ShowOptionsScreen;
    procedure Execute;
  end;

implementation

uses
  GameMenuScreen,
  GameLevelCodeScreen,
  GamePreviewScreen,
  GamePostviewScreen,
  GameOptionsScreen,
  GameWindow;

{ TAppController }

constructor TAppController.Create(aOwner: TComponent);
var
  Fn: string;
begin
  inherited;
  fGameParams := TDosGameParams.Create;
  fGameParams.Directory := LemmingsPath;
  fGameParams.MainDatFile := LemmingsPath + 'main.dat';
  fGameParams.Renderer := TRenderer.Create;
  fGameParams.Level := Tlevel.Create(nil);
  fGameParams.Style := AutoCreateStyle(fGameParams.Directory);
//  fGameParams.Style.LevelSystem as TBaseDosLevelSystem
  fGameParams.GraphicSet := fGameParams.Style.CreateGraphicSet as TBaseDosGraphicSet;
  fGameParams.NextScreen := gstMenu;
  fGameParams.SoundOptions := [gsoSound, gsoMusic];

  Fn := ReplaceFileExt(Application.Exename, '.ini');//' AppPath + 'LemmixPlayer.ini';
  if FileExists(Fn) then
    fGameParams.LoadFromIniFile(Fn);

  if fGameParams.Style.LevelSystem is TBaseDosLevelSystem then
    TBaseDosLevelSystem(fGameParams.Style.LevelSystem).LookForLVL := fGameParams.LookForLVLFiles;

end;

destructor TAppController.Destroy;
var
  Fn: string;
begin
  Fn := ReplaceFileExt(Application.Exename, '.ini');//' AppPath + 'LemmixPlayer.ini';
//  Fn := AppPath + 'LemmixPlayer.ini';
  fGameParams.SaveToIniFile(Fn);

  fGameParams.Renderer.Free;
  fGameParams.Level.Free;
  fGameParams.Style.Free;
  fGameParams.GraphicSet.Free;
  fGameParams.Free;
  inherited;
end;

procedure TAppController.Execute;
{-------------------------------------------------------------------------------
  Main screen-loop. Every screen returns its nextscreen (if he knows) in the
  GameParams
-------------------------------------------------------------------------------}
var
  NewScreen: TGameScreenType;
begin
  while fGameParams.NextScreen <> gstExit do
  begin
    NewScreen := fGameParams.NextScreen; // save
    fGameParams.NextScreen := gstUnknown; // reset

    case NewScreen of
      gstMenu      : ShowMenuScreen;
      gstPreview   : ShowPreviewScreen;
      gstPlay      : ShowPlayScreen;
      gstPostview  : ShowPostviewScreen;
      gstNavigation: ShowOptionsScreen;
      gstLevelCode : ShowLevelCodeScreen;
    else
      Exit;
    end;
  end;
end;

procedure TAppController.ShowLevelCodeScreen;
var
  F: TGameLevelCodeScreen;
begin
  F := TGameLevelCodeScreen.Create(nil);
  try
    F.ShowScreen(fGameParams);
  finally
    F.Free;
  end;
end;

procedure TAppController.ShowMenuScreen;
var
  F: TGameMenuScreen;
begin
  F := TGameMenuScreen.Create(nil);
  try
    F.ShowScreen(fGameParams);
  finally
    F.Free;
  end;
end;

procedure TAppController.ShowOptionsScreen;
var
  F: TGameOptionsScreen;
begin
  F := TGameOptionsScreen.Create(nil);
  try
    F.ShowScreen(fGameParams);
  finally
    F.Free;
  end;
end;

procedure TAppController.ShowPlayScreen;
var
  F: TGameWindow;
begin
  F := TGameWindow.Create(nil);
  try
    F.ShowScreen(fGameParams);
  finally
    F.Free;
  end;
end;

procedure TAppController.ShowPostviewScreen;
var
  F: TGamePostviewScreen;
begin
  F := TGamePostviewScreen.Create(nil);
  try
    F.ShowScreen(fGameParams);
  finally
    F.Free;
  end;
end;

procedure TAppController.ShowPreviewScreen;
var
  F: TGamePreviewScreen;
begin
  // this must be loaded before entering the previewscreen
//  with GameParams, Info do
  //  Style.LevelSystem.LoadSingleLevel(dPack, dSection, dLevel, Level);
//      Style.LevelSystem.LoadSingleLevel(0, 3, 0, Level);

  F := TGamePreviewScreen.Create(nil);
  try
    //F.GameParams := GameParams;
    //F.MainDatFile := GameParams.Directory + 'main.dat';
//    F.BuildScreen;
  //  F.ShowModal;
    F.ShowScreen(fGameParams);
  finally
    F.Free;
  end;
end;

end.
                  (*
procedure TGameMenuScreen.ShowLevelCodeScreen;
var
  F: TGameLevelCodeScreen;
begin
  F := TGameLevelCodeScreen.Create(nil);
  try
//    F.MainDatFile := Self.MainDatFile;
    F.BuildScreen;
    F.ShowModal;
  finally
    F.Free;
  end;

end;

