{$include lem_directives.inc}

unit GameControl;

{-------------------------------------------------------------------------------
  The gamecontrol class is in fact just a global var which is passed through as
  a parameter to all screens.
-------------------------------------------------------------------------------}

interface

uses
  Classes,
  GR32, GR32_Image,
  UTools,
  LemCore, LemTypes, LemLevel, LemDosStyle, LemGraphicSet, LemDosGraphicSet,
  LemLevelSystem, LemRendering;

type
  TGameResultsRec = record
    gSuccess            : Boolean; // level played successfully?
    gCheated            : Boolean; // level cheated?
    gCount              : Integer; // number
    gToRescue           : Integer;
    gRescued            : Integer;
    gTarget             : Integer;
    gDone               : Integer;
    gTimeIsUp           : Boolean;
  end;

type
  TGameScreenType = (
    gstUnknown,
    gstMenu,
    gstPreview,
    gstPlay,
    gstPostview,
    gstLevelCode,
    gstSounds,
    gstExit,
    gstNavigation
  );

type
  // how do we load the level
  TWhichLevel = (
    wlFirst,
    wlFirstSection,
    wlLevelCode,
    wlNext,
    wlSame,
    wlCongratulations
  );

type
  TGameSoundOption = (
    gsoSound,
    gsoMusic
  );
  TGameSoundOptions = set of TGameSoundOption;

type
  TMiscOption = (
    moGradientBridges,  // 0
    moFastForward,      // 1
    moSaveState,        // 2
    moHyperJumps,       // 3
    moCheatCodes,       // 4
    moShowParticles,    // 5
    moLookForLVLFiles,  // 6
//    moUseSystemCursor,  // 7
    mo8, mo9, mo10, mo11, mo12, mo13, mo14, mo15,
    mo16, mo17, mo18, mo19, mo20, mo21, mo22, mo23,
    mo24, mo25, mo26, mo27, mo28, mo29, mo30, mo31
  );

  TMiscOptions = set of TMiscOption;

const
  DEF_MISCOPTIONS = [
    moGradientBridges,
    moFastForward,
    moSaveState,
    moHyperJumps,
//    moCheatCodes,
    moShowParticles
//    moLookForLVLFiles
//    moUseSystemCursor
  ];

type             (*
  TOptionsRec = packed record
    oSignature   : array[0..2] of Char; // LOF LemmingsOptions File
    oVersion     : Byte; // version of this record
    oRecordSize  : Integer;
    oOptions     : TMiscOptions; // 32 bits
    oSoundOptions: TGameSoundOptions; // 8 bits
  end;         *)

  TDosGameParams = class(TPersistent)
  private
    fDirectory    : string;
    function GetCheatCodesEnabled: Boolean;
    procedure SetCheatCodesEnabled(Value: Boolean);
    function GetMusicEnabled: Boolean;
    function GetSoundEnabled: Boolean;
    procedure SetMusicEnabled(Value: Boolean);
    procedure SetSoundEnabled(Value: Boolean);
    function GetShowParticles: Boolean;
    procedure SetShowParticles(const Value: Boolean);
    function GetLookForLVLFiles: Boolean;
    procedure SetLookForLVLFiles(Value: Boolean);
    function GetUseSystemCursor: Boolean;
    procedure SetUseSystemCursor(const Value: Boolean);
  public
    // this is initialized by appcontroller
    MainDatFile  : string;

    Info         : TDosGamePlayInfoRec;
    WhichLevel   : TWhichLevel;

    SoundOptions : TGameSoundOptions;

    Level        : TLevel;
    Style        : TBaseDosLemmingStyle;
    GraphicSet   : TBaseDosGraphicSet;
    Renderer     : TRenderer;

    // this is initialized by the window in which the game will be played
    TargetBitmap : TBitmap32;

    // this is initialized by the game
    GameResult: TGameResultsRec;

    // this is set by the individual screens when closing (if they know)
    NextScreen: TGameScreenType;

    // resource vars
    LemDataInResource   : Boolean;
    LemDataOnDisk       : Boolean;
    LemSoundsInResource : Boolean;
    LemSoundsOnDisk     : Boolean;
    LemMusicInResource  : Boolean;
    LemMusicOnDisk      : Boolean;

    // cheat
//    Cheatmode: Boolean; // levelcode screen
    MiscOptions         : TMiscOptions;

    fZoomFactor          : Integer;
    // set if replay loaded in postviewscreen
    //StartupWithReplay   : Boolean;

    constructor Create;
    procedure LoadFromIniFile(const aFileName: string);
    procedure SaveToIniFile(const aFileName: string);


    property Directory: string read fDirectory write fDirectory;
  published
  { easy readable published props for streaming to inifile }
    //property HyperJumpsEnabled: Boolean read GetHyperJumpsEnabled write SetHyperJumpsEnabled;
    property CheatCodesEnabled: Boolean read GetCheatCodesEnabled write SetCheatCodesEnabled;
    property MusicEnabled: Boolean read GetMusicEnabled write SetMusicEnabled;
    property SoundEnabled: Boolean read GetSoundEnabled write SetSoundEnabled;
    property ShowParticles: Boolean read GetShowParticles write SetShowParticles;
    property LookForLVLFiles: Boolean read GetLookForLVLFiles write SetLookForLVLFiles;
//    property UseSystemCursor: Boolean read GetUseSystemCursor write SetUseSystemCursor;

  { zoom }
    property ZoomFactor: Integer read fZoomFactor write fZoomFactor;
  end;


implementation

{ TDosGameParams }

constructor TDosGameParams.Create;
begin
  inherited Create;

  MiscOptions := DEF_MISCOPTIONS;

  {$ifdef develop}
    Include(MiscOptions, moCheatCodes);
  {$endif}

  {$ifdef resourcelemdata}
    LemDataInResource := True;
  {$endif}

  {$ifdef resourcelemsounds}
    LemSoundsInResource := True;
  {$endif}

  {$ifdef resourcelemmusic}
    LemMusicInResource := True;
  {$endif}

  {.$ifdef resourcebassmod}
  {.$endif}



end;

function TDosGameParams.GetCheatCodesEnabled: Boolean;
begin
  Result := moCheatCodes in MiscOptions;
end;

function TDosGameParams.GetMusicEnabled: Boolean;
begin
  Result := gsoMusic in SoundOptions;
end;

function TDosGameParams.GetShowParticles: Boolean;
begin
  Result := moShowParticles in MiscOptions;
end;

function TDosGameParams.GetSoundEnabled: Boolean;
begin
  Result := gsoSound in SoundOptions;
end;

function TDosGameParams.GetLookForLVLFiles: Boolean;
begin
  Result := moLookForLVLFiles in MiscOptions;
end;

procedure TDosGameParams.LoadFromIniFile(const aFileName: string);
begin
  IniToObject(aFileName, 'GameSettings', Self, False);
end;

procedure TDosGameParams.SaveToIniFile(const aFileName: string);
begin
  ObjectToIni(aFileName, 'GameSettings', Self, False);
end;

procedure TDosGameParams.SetCheatCodesEnabled(Value: Boolean);
begin
  case Value of
    False: Exclude(MiscOptions, moCheatCodes);
    True:  Include(MiscOptions, moCheatCodes);
  end;
end;

procedure TDosGameParams.SetMusicEnabled(Value: Boolean);
begin
  case Value of
    False: Exclude(SoundOptions, gsoMusic);
    True:  Include(SoundOptions, gsoMusic);
  end;
end;

procedure TDosGameParams.SetShowParticles(const Value: Boolean);
begin
  case Value of
    False: Exclude(MiscOptions, moShowParticles);
    True:  Include(MiscOptions, moShowParticles);
  end;
end;

procedure TDosGameParams.SetSoundEnabled(Value: Boolean);
begin
  case Value of
    False: Exclude(SoundOptions, gsoSound);
    True:  Include(SoundOptions, gsoSound);
  end;
end;

procedure TDosGameParams.SetLookForLVLFiles(Value: Boolean);
begin
  case Value of
    False: Exclude(MiscOptions, moLookForLVLFiles);
    True:  Include(MiscOptions, moLookForLVLFiles);
  end;
end;

function TDosGameParams.GetUseSystemCursor: Boolean;
begin
//  Result := moUseSystemCursor in MiscOptions;
end;

procedure TDosGameParams.SetUseSystemCursor(const Value: Boolean);
begin
  {case Value of
    False: Exclude(MiscOptions, moUseSystemCursor);
    True:  Include(MiscOptions, moUseSystemCursor);
  end;}
end;

end.

