{$include lem_directives.inc}

unit GamePreviewScreen;

interface

uses
  Windows, Classes, Controls, Graphics, SysUtils,
  GR32, GR32_Image, GR32_Layers,
  UMisc,
  LemCore, LemStrings, LemDosStructures, LemRendering, LemLevelSystem,
  GameControl, GameBaseScreen, GameWindow;

type
  TGamePreviewScreen = class(TGameBaseScreen)
  private
    procedure Form_KeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure Form_MouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure Img_MouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer; Layer: TCustomLayer);
    function GetScreenText: string;
  protected
  public
    constructor Create(aOwner: TComponent); override;
    destructor Destroy; override;
    procedure BuildScreen; override;
    procedure PrepareGameParams(Params: TDosGameParams); override;
  end;

implementation

uses LemGraphicSet, LemDosGraphicSet, LemLevel, FBaseDosForm;

{ TGamePreviewScreen }

procedure TGamePreviewScreen.BuildScreen;
var
  Inf: TRenderInfoRec;
  Mainpal: TArrayOfColor32;
  Temp, W: TBitmap32;
  DstRect: TRect;
begin
  Assert(GameParams <> nil);

  ScreenImg.BeginUpdate;
  try
    MainPal := GetDosMainMenuPaletteColors32;
    InitializeImageSizeAndPosition(640, 350);
    ExtractBackGround;
    ExtractPurpleFont;


    // prepare the renderer, this is a little bit shaky (wrong place)
    with GameParams do
    begin
      with GraphicSet do
      begin
        ClearMetaData;
        ClearData;
        GraphicSetId := Level.Info.GraphicSet;
        GraphicSetIdExt := Level.Info.GraphicSetEx;
        MetaInfoFile := GameParams.Directory + 'ground' + i2s(GraphicSetId) + 'o.dat';
        GraphicFile := GameParams.Directory + 'vgagr' + i2s(GraphicSetId) + '.dat';
        if GraphicSetIdExt = 0 then
          GraphicExtFile := ''
        else
          GraphicExtFile := GameParams.Directory + 'vgaspec' + i2s(GraphicSetIdExt - 1) + '.dat';
        ReadMetaData;
        ReadData;
      end;
      Inf.Level:=Level;
      Inf.GraphicSet := Graphicset;
      Renderer.PrepareGameRendering(Inf);
    end;
    


    Temp := TBitmap32.Create;
    W := TBitmap32.Create;
    try
      Temp.SetSize(640, 350);
      Temp.Clear(0);
      // draw level preview
      W.SetSize(GAME_BMPWIDTH, 160);   
      W.Clear(0);
//      W.ResamplerClassName := 'TLinearResampler';//DraftResampler';
//      W.ResamplerClassName := 'TDraftResampler';
      GameParams.Renderer.RenderWorld(W, True);
      DstRect := Rect(0, 0, 400, 40); // div 4
      RectMove(DstRect, 120, 20); // set location
      W.DrawTo(Temp, DstRect, W.BoundsRect);
      // draw background
      TileBackgroundBitmap(0, 78, Temp);
      // draw text
      DrawPurpleText(Temp, GetScreenText, 0, 80);
      ScreenImg.Bitmap.Assign(Temp);
    finally
      W.Free;
      Temp.Free;
    end;


  finally
    ScreenImg.EndUpdate;
  end;
end;

constructor TGamePreviewScreen.Create(aOwner: TComponent);
begin
  inherited;
  OnKeyDown := Form_KeyDown;
  OnMouseDown := Form_MouseDown;
  ScreenImg.OnMouseDown := Img_MouseDown;
end;

destructor TGamePreviewScreen.Destroy;
begin

  inherited;
end;

procedure TGamePreviewScreen.Form_KeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
  case Key of
    VK_ESCAPE: CloseScreen(gstMenu);
    VK_RETURN: CloseScreen(gstPlay);
  end;
end;

procedure TGamePreviewScreen.Form_MouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if Button = mbLeft then
    CloseScreen(gstPlay);
end;

procedure TGamePreviewScreen.Img_MouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer;
  Layer: TCustomLayer);
begin
  if Button = mbLeft then
    CloseScreen(gstPlay);
end;


function TGamePreviewScreen.GetScreenText: string;
var
  Perc: string;
begin
  Assert(GameParams <> nil);

//  with GameParams.Level.Info do
  begin
    Perc := PadR(i2s(Percentage(GameParams.Level.Info.LemmingsCount,
                                GameParams.Level.Info.RescueCount)) + '%', 4);
    Result := Format(SPreviewString,
                [GameParams.Info.dLevel + 1, // humans read 1-based
                 Trim(GameParams.Level.Info.Title),
                 GameParams.Level.Info.LemmingsCount,
                 Perc,
                 GameParams.Level.Info.ReleaseRate,
                 GameParams.Level.Info.TimeLimit,
                 GameParams.Info.dSectionName
               ]);
  end;
end;

procedure TGamePreviewScreen.PrepareGameParams(Params: TDosGameParams);
//var
  //Inf: TDosGamePlayInfoRec;
begin
  inherited;
  with Params, Info do
  begin

    case WhichLevel of
      wlFirst: Style.LevelSystem.FindFirstLevel(Info);
      wlFirstSection: Style.LevelSystem.FindFirstLevel(Info);
      wlLevelCode: Style.LevelSystem.FindLevel(Info);
      wlNext: Style.LevelSystem.FindNextLevel(Info);
      wlSame: Style.LevelSystem.FindLevel(Info);
    end;

    (*
    if not GameResult.gValid then
    begin
      if not Info.dValid then
        Style.LevelSystem.FindFirstLevel(Info)
      else
        Style.LevelSystem.FindLevel(Info) // used to fill sectionname
    end
    else if GameResult.gSuccess then
    begin
      Style.LevelSystem.FindNextLevel(Info);
    end;
    ClearGameResult;
      *)

    Style.LevelSystem.LoadSingleLevel(dPack, dSection, dLevel, Level);


    WhichLevel := wlSame;

//    Style.LevelSystem.LoadSingleLevel(0, 3, 1, Level);

  end;
end;

end.

