{$include lem_directives.inc}

unit GameSound;

interface

uses
  Windows, Classes, Contnrs, SysUtils,
//  WavePlayer,
  LemTypes;

type
  TAbstractSound = class
  private
    fStream: TMemoryStream;
    fResourceDataType: TLemDataType;
  protected
  public
    constructor Create;
    destructor Destroy; override;
    procedure LoadFromFileName(const aFileName: string);
    procedure LoadFromStream(aStream: TStream);
//    procedure Play; virtual; abstract;
//    procedure Stop; virtual; abstract;
  end;

  TSound = class(TAbstractSound)
  private
  protected
  public
    constructor Create;
//    procedure Play; override;
//    procedure Stop; override;
  end;

  TMusic = class(TAbstractSound)
  private
  protected
  public
    constructor Create;
//    procedure Play; override;
//    procedure Stop; override;
  end;

  TSoundList = class(TObjectList)
  private
    function GetItem(Index: Integer): TSound;
  protected
  public
    function Add(Item: TSound): Integer;
    procedure Insert(Index: Integer; Item: TSound);
    property Items[Index: Integer]: TSound read GetItem; default;
  published
  end;


  TMusicList = class(TObjectList)
  private
    function GetItem(Index: Integer): TMusic;
  protected
  public
    function Add(Item: TMusic): Integer;
    procedure Insert(Index: Integer; Item: TMusic);
    property Items[Index: Integer]: TMusic read GetItem; default;
  published
  end;

  TSoundMgr = class
  private
    fMusics: TMusicList;
    fSounds: TSoundList;
  protected
  public
    constructor Create;
    destructor Destroy; override;
  { sounds }
    function AddSoundFromFileName(const aFileName: string): Integer;
    procedure PlaySound(Index: Integer);
    procedure StopSound(Index: Integer);
  { musics }
    function AddMusicFromFileName(const aFileName: string): Integer;
    procedure PlayMusic(Index: Integer);
    procedure StopMusic(Index: Integer);

    property Musics: TMusicList read fMusics;
    property Sounds: TSoundList read fSounds;
  end;

(*  TWavePlayerEx = class(TWavePlayer)
  private
//    Ix: Integer;
  protected
  public
  end;
  *)

  TSoundSystem = class
  private
    //fWavePlayer: TWavePlayerEx;//array[0..2] of TWavePlayerEx;
//    procedure WavePlayer_Finished(Sender: TObject);
  protected
  public
    constructor Create;
    destructor Destroy; override;
    procedure InitWav;
    procedure PlayWav(S: TMemoryStream);
    procedure StopWav(S: TMemoryStream);

    procedure PlayMusic(S: TMemoryStream);
    procedure StopMusic(S: TMemoryStream);

  end;

implementation

uses
  MMSystem,
  GameModPlay;

var
  _SoundSystem: TSoundSystem;

function SoundSystem: TSoundSystem;
begin
  if _SoundSystem = nil then
    _SoundSystem := TSoundSystem.Create;
  Result := _SoundSystem;
end;

{ TAbstractSound }

constructor TAbstractSound.Create;
begin
  inherited Create;
  fStream := TMemoryStream.Create;
end;

destructor TAbstractSound.Destroy;
begin
  fStream.Free;
  inherited;
end;

procedure TAbstractSound.LoadFromFileName(const aFileName: string);
{-------------------------------------------------------------------------------
  Loads from file or archive file in resource
-------------------------------------------------------------------------------}
var
  F: TStream;
begin
  F := CreateDataStream(aFileName, fResourceDataType);
  try
    LoadFromStream(F);
  finally
    F.Free;
  end;
end;

procedure TAbstractSound.LoadFromStream(aStream: TStream);
begin
  fStream.LoadFromStream(aStream);
end;

{ TSound }

constructor TSound.Create;
begin
  inherited Create;
  fResourceDataType := ldtSound;
end;

{ TMusic }

constructor TMusic.Create;
begin
  inherited;
  fResourceDataType := ldtMusic;
end;

(*
procedure TSound.Play;
begin
  SoundSystem.PlayWav(fStream);
end;

procedure TSound.Stop;
begin
  SoundSystem.StopWav(fStream);
end;

procedure TMusic.Play;
begin
  SoundSystem.PlayMusic(fStream);
end;

procedure TMusic.Stop;
begin
  SoundSystem.StopMusic(fStream);
end;
*)

{ TSoundList }

function TSoundList.Add(Item: TSound): Integer;
begin
  Result := inherited Add(Item);
end;

function TSoundList.GetItem(Index: Integer): TSound;
begin
  Result := inherited Get(Index);
end;

procedure TSoundList.Insert(Index: Integer; Item: TSound);
begin
  inherited Insert(Index, Item);
end;

{ TMusicList }

function TMusicList.Add(Item: TMusic): Integer;
begin
  Result := inherited Add(Item);
end;

function TMusicList.GetItem(Index: Integer): TMusic;
begin
  Result := inherited Get(Index);
end;

procedure TMusicList.Insert(Index: Integer; Item: TMusic);
begin
  inherited Insert(Index, Item);
end;

{ TSoundMgr }

function TSoundMgr.AddMusicFromFileName(const aFileName: string): Integer;
var
  M: TMusic;
begin
  M := TMusic.Create;
  Result := Musics.Add(M);
  M.LoadFromFileName(aFileName);
end;

function TSoundMgr.AddSoundFromFileName(const aFileName: string): Integer;
var
  S: TSound;
begin
  S := TSound.Create;
  Result := Sounds.Add(S);
  S.LoadFromFileName(aFileName);
end;

constructor TSoundMgr.Create;
begin
  inherited Create;
  fSounds := TSoundList.Create;
  fMusics := TMusicList.Create;
end;

destructor TSoundMgr.Destroy;
begin
  StopSound(0);
  StopMusic(0);
  fSounds.Free;
  fMusics.Free;
  inherited Destroy;
end;

procedure TSoundMgr.PlaySound(Index: Integer);
begin
  if Index >= 0 then
    if Index < Sounds.Count then
      //Sounds[Index].Play;
      SoundSystem.PlayWav(Sounds[Index].fStream);
end;

procedure TSoundMgr.PlayMusic(Index: Integer);
begin
  if Index >= 0 then
    if Index < Musics.Count then
    //  Musics[Index].Play;
      SoundSystem.PlayMusic(Musics[Index].fStream)
end;

procedure TSoundMgr.StopSound(Index: Integer);
begin
//  if Index >= 0 then
  //  if Index <
  SoundSystem.StopWav(nil);
end;

procedure TSoundMgr.StopMusic(Index: Integer);
begin
  if Index >= 0 then
    if Index < Musics.Count then
      //Musics[Index].Stop;
      SoundSystem.StopMusic(nil);
end;

{ TSoundSystem }

constructor TSoundSystem.Create;
//var
  //i: Integer;
begin
  inherited Create;

  // init wav (to avoid small delay when starting the first game)
  InitWav;

//  for i := 0 to 2 do
  begin
//    fWavePlayer := TWavePlayerEx.Create;
//    fWavePlayers[i].Ix := i;
  //  fWavePlayers[i].OnFinishedPlayback := WavePlayer_Finished;
  end;

  // init mod
  // Check that BASSMOD 2.0 was loaded
  if BASSMOD_GetVersion <> 2 {MAKELONG(2, 0)} then
  begin
    //Error('BASSMOD version 2.0 was not loaded');
    //self.Destroy;
    //Halt;
  end;
  // setup output - default device, 44100hz, stereo, 16 bits
  if not BASSMOD_Init(-1, 44100, 0) then
  begin
    //Error('Can''t initialize device');
    BASSMOD_Free;
    //self.Destroy;
    //Halt;
  end;

end;

destructor TSoundSystem.Destroy;
//var
  //i: Integer;
begin
  BASSMOD_Free;
//  for i := 0 to 2 do
//    fWavePlayer.Free;
  inherited;
end;

procedure TSoundSystem.InitWav;
begin
  PlaySound(nil, HINSTANCE, SND_ASYNC + SND_MEMORY);
end;

procedure TSoundSystem.PlayMusic(S: TMemoryStream);
const
  MusicFlags = BASS_MUSIC_LOOP or BASS_MUSIC_RAMPS or BASS_MUSIC_SURROUND or
               BASS_MUSIC_POSRESET;
begin
  BASSMOD_MusicFree;
  if S.Size > 0 then
    if BASSMOD_MusicLoad(True, S.Memory, 0, 0, MusicFlags) then
      BASSMOD_MusicPlay
end;

procedure TSoundSystem.PlayWav(S: TMemoryStream);
//var
  //i: Integer;
begin
(*
//  for i := 0 to 2 do
  begin
    with fWavePlayer do
    begin
//      if State = wpPlaying then
  //      Stop;
//      if State <> wpPlaying then
      begin
        Source := S;
        Play;
  //      Exit;
      end;
    end;
  end; *)
  PlaySound(S.Memory, HINSTANCE, SND_ASYNC + SND_MEMORY);
end;

procedure TSoundSystem.StopMusic(S: TMemoryStream);
begin
  BASSMOD_MusicFree;
end;

procedure TSoundSystem.StopWav(S: TMemoryStream);
begin
  PlaySound(nil, HINSTANCE, SND_ASYNC + SND_MEMORY);
end;

initialization
finalization
  _SoundSystem.Free;
  _SoundSystem := nil;
end.


